/**
 * Generated by Apache Royale Compiler from org/apache/royale/net/URLStream.as
 * org.apache.royale.net.URLStream
 *
 * @fileoverview
 *
 * @suppress {checkTypes|accessControls}
 */

goog.provide('org.apache.royale.net.URLStream');

goog.require('org.apache.royale.events.DetailEvent');
goog.require('org.apache.royale.events.Event');
goog.require('org.apache.royale.events.EventDispatcher');
goog.require('org.apache.royale.events.ProgressEvent');
goog.require('org.apache.royale.net.HTTPUtils');
goog.require('org.apache.royale.utils.BinaryData');
goog.require('org.apache.royale.utils.Endian');
goog.require('org.apache.royale.utils.Language');



/**
 * constructor
 * @constructor
 * @extends {org.apache.royale.events.EventDispatcher}
 */
org.apache.royale.net.URLStream = function() {
  
  this.endian = org.apache.royale.utils.Endian.BIG_ENDIAN;
  org.apache.royale.net.URLStream.base(this, 'constructor');
};
goog.inherits(org.apache.royale.net.URLStream, org.apache.royale.events.EventDispatcher);


/**
 * Prevent renaming of class. Needed for reflection.
 */
goog.exportSymbol('org.apache.royale.net.URLStream', org.apache.royale.net.URLStream);


/**
 * @export
 * @type {number}
 */
org.apache.royale.net.URLStream.prototype.bytesLoaded = 0;


/**
 * @export
 * @type {number}
 */
org.apache.royale.net.URLStream.prototype.bytesTotal = 0;


/**
 * loads the request
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @export
 * @param {org.apache.royale.net.URLRequest} urlRequest
 */
org.apache.royale.net.URLStream.prototype.load = function(urlRequest) {
  this.requestStatus = 0;
  this.xhr = new XMLHttpRequest();
  this.xhr.open(urlRequest.method, urlRequest.url);
  this.xhr.responseType = "arraybuffer";
  this.xhr.addEventListener("readystatechange", org.apache.royale.utils.Language.closure(this.xhr_onreadystatechange, this, 'xhr_onreadystatechange'), false);
  this.xhr.addEventListener("progress", org.apache.royale.utils.Language.closure(this.xhr_progress, this, 'xhr_progress'), false);
  var /** @type {boolean} */ contentTypeSet = false;
  for (var /** @type {number} */ i = 0; i < urlRequest.requestHeaders.length; i++) {
    var /** @type {org.apache.royale.net.URLRequestHeader} */ header = urlRequest.requestHeaders[i];
    if (header.name.toLowerCase() == "content-type") {
      contentTypeSet = true;
    }
    this.xhr.setRequestHeader(header.name, header.value);
  }
  if (!contentTypeSet) {
    this.xhr.setRequestHeader("Content-type", urlRequest.contentType);
  }
  var /** @type {Object} */ requestData = org.apache.royale.utils.Language.is(urlRequest.data, org.apache.royale.utils.BinaryData) ? org.apache.royale.utils.Language.as(urlRequest.data, org.apache.royale.utils.BinaryData).data : org.apache.royale.net.HTTPUtils.encodeUrlVariables(urlRequest.data);
  this.send(requestData);
  
};


/**
 * send is a protected function in js so a subclass can attach an upload listener
 * without rewriting the whole load() function
 * @protected
 * @param {Object} requestData
 */
org.apache.royale.net.URLStream.prototype.send = function(requestData) {
  this.xhr.send(requestData);
};


/**
 *  Download is progressing (JS only).
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @private
 * @param {Object} error
 */
org.apache.royale.net.URLStream.prototype.xhr_progress = function(error) {
  var /** @type {org.apache.royale.events.ProgressEvent} */ progEv = new org.apache.royale.events.ProgressEvent(org.apache.royale.events.ProgressEvent.PROGRESS);
  progEv.current = this.bytesLoaded = Number(error.loaded);
  progEv.total = this.bytesTotal = Number(error.total);
  this.dispatchEvent(progEv);
  if (this.onProgress)
    this.onProgress(this);
};


/**
 *  HTTP status change (JS only).
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @private
 * @param {*} error
 */
org.apache.royale.net.URLStream.prototype.xhr_onreadystatechange = function(error) {
  this.setStatus(this.xhr.status);
  if (this.xhr.readyState != 4)
    return;
  if (this.xhr.status == 0) {
    this.dispatchEvent(new org.apache.royale.events.DetailEvent("communicationError"));
    if (this.onError)
      this.onError(this);
  } else if (this.xhr.status < 200) {
    this.dispatchEvent(new org.apache.royale.events.DetailEvent("communicationError", false, false, "" + this.requestStatus));
    if (this.onError)
      this.onError(this);
  } else if (this.xhr.status < 300) {
    this.dispatchEvent(new org.apache.royale.events.Event("complete"));
    if (this.onComplete)
      this.onComplete(this);
  } else {
    this.dispatchEvent(new org.apache.royale.events.DetailEvent("communicationError", false, false, "" + this.requestStatus));
    if (this.onError)
      this.onError(this);
  }
  this.cleanupCallbacks();
};


/**
 *  Set the HTTP request status.
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @private
 * @param {number} value
 */
org.apache.royale.net.URLStream.prototype.setStatus = function(value) {
  if (value != this.requestStatus) {
    this.requestStatus = value;
    this.dispatchEvent(new org.apache.royale.events.DetailEvent("httpStatus", false, false, "" + value));
    if (this.onStatus)
      this.onStatus(this);
  }
};


/**
 *  Abort an connection.
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @export
 */
org.apache.royale.net.URLStream.prototype.close = function() {
  
  this.xhr.abort();
  this.cleanupCallbacks();
};


/**
 * @export
 * @type {number}
 */
org.apache.royale.net.URLStream.prototype.requestStatus = 0;


/**
 * @export
 * @type {string}
 */
org.apache.royale.net.URLStream.prototype.endian;


/**
 *  Cleanup all callbacks.
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @protected
 */
org.apache.royale.net.URLStream.prototype.cleanupCallbacks = function() {
  this.onComplete = null;
  this.onError = null;
  this.onProgress = null;
  this.onStatus = null;
};


/**
 * @export
 * @type {Function}
 */
org.apache.royale.net.URLStream.prototype.onComplete;


/**
 * @export
 * @type {Function}
 */
org.apache.royale.net.URLStream.prototype.onError;


/**
 * @export
 * @type {Function}
 */
org.apache.royale.net.URLStream.prototype.onProgress;


/**
 * @export
 * @type {Function}
 */
org.apache.royale.net.URLStream.prototype.onStatus;


/**
 *  Convenience function for complete event to allow chaining.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @export
 * @param {Function} callback
 * @return {org.apache.royale.net.URLStream}
 */
org.apache.royale.net.URLStream.prototype.complete = function(callback) {
  this.onComplete = callback;
  return this;
};


/**
 *  Convenience function for error event to allow chaining.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @export
 * @param {Function} callback
 * @return {org.apache.royale.net.URLStream}
 */
org.apache.royale.net.URLStream.prototype.error = function(callback) {
  this.onError = callback;
  return this;
};


/**
 *  Convenience function for progress event to allow chaining.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @export
 * @param {Function} callback
 * @return {org.apache.royale.net.URLStream}
 */
org.apache.royale.net.URLStream.prototype.progress = function(callback) {
  this.onProgress = callback;
  return this;
};


/**
 *  Convenience function for status event to allow chaining.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @export
 * @param {Function} callback
 * @return {org.apache.royale.net.URLStream}
 */
org.apache.royale.net.URLStream.prototype.status = function(callback) {
  this.onStatus = callback;
  return this;
};


org.apache.royale.net.URLStream.prototype.get__response = function() {
  return new org.apache.royale.utils.BinaryData(org.apache.royale.utils.Language.as(this.xhr.response, ArrayBuffer));
  
};


Object.defineProperties(org.apache.royale.net.URLStream.prototype, /** @lends {org.apache.royale.net.URLStream.prototype} */ {
/**
  * @export
  * @type {org.apache.royale.utils.BinaryData} */
response: {
get: org.apache.royale.net.URLStream.prototype.get__response}}
);


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
org.apache.royale.net.URLStream.prototype.ROYALE_CLASS_INFO = { names: [{ name: 'URLStream', qName: 'org.apache.royale.net.URLStream', kind: 'class' }] };



/**
 * Reflection
 *
 * @return {Object.<string, Function>}
 */
org.apache.royale.net.URLStream.prototype.ROYALE_REFLECTION_INFO = function () {
  return {
    variables: function () {
      return {
        'bytesLoaded': { type: 'uint'},
        'bytesTotal': { type: 'uint'},
        'requestStatus': { type: 'int'},
        'endian': { type: 'String'},
        'onComplete': { type: 'Function'},
        'onError': { type: 'Function'},
        'onProgress': { type: 'Function'},
        'onStatus': { type: 'Function'}
      };
    },
    accessors: function () {
      return {
        'response': { type: 'org.apache.royale.utils.BinaryData', access: 'readonly', declaredBy: 'org.apache.royale.net.URLStream'}
      };
    },
    methods: function () {
      return {
        'URLStream': { type: '', declaredBy: 'org.apache.royale.net.URLStream'},
        'load': { type: 'void', declaredBy: 'org.apache.royale.net.URLStream', parameters: function () { return [  { index: 1, type: 'org.apache.royale.net.URLRequest', optional: false } ]; }},
        'close': { type: 'void', declaredBy: 'org.apache.royale.net.URLStream'},
        'complete': { type: 'org.apache.royale.net.URLStream', declaredBy: 'org.apache.royale.net.URLStream', parameters: function () { return [  { index: 1, type: 'Function', optional: false } ]; }},
        'error': { type: 'org.apache.royale.net.URLStream', declaredBy: 'org.apache.royale.net.URLStream', parameters: function () { return [  { index: 1, type: 'Function', optional: false } ]; }},
        'progress': { type: 'org.apache.royale.net.URLStream', declaredBy: 'org.apache.royale.net.URLStream', parameters: function () { return [  { index: 1, type: 'Function', optional: false } ]; }},
        'status': { type: 'org.apache.royale.net.URLStream', declaredBy: 'org.apache.royale.net.URLStream', parameters: function () { return [  { index: 1, type: 'Function', optional: false } ]; }}
      };
    }
  };
};